@@ -30,10 +30,13 @@
 #include <ctype.h>
 #include <string.h>
 #include <errno.h>
+#ifdef HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif /* HAVE_DMALLOC_H */
 
 #include <hash.h>
 #include <getopttools.h>
-#include <dirname.h>
+#include <urldirname.h>
 #include <split.h>
 #include <salloc.h>
 #include <uri.h>
@@ -82,6 +85,7 @@ static webbase_url_t* mirror_1(crawl_context_t* context, char* url);
 static webbase_url_t* mirror_2(crawl_context_t* context, char* url);
 static void mirror_scheme(crawl_context_t* context, webbase_url_t* webbase_url, uri_t* url_object, char* url);
 static void ftp_body_parse(webbase_url_t* webbase_url, uri_t* url_object, char* path);
+static void mirror_ftp_free();
 static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_t* url_object, char* url);
 static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri_t* url_object, char* url);
 static webbase_url_t* mirror_location(crawl_context_t* context, webbase_url_t* webbase_url);
@@ -676,7 +680,7 @@ static void hp_load_mirror(char* paramsp, webbase_url_start_t* start)
   context.params = params;
   context.base = params->base;
   context.start = start;
-  context.home = strdup(dirname(start->url));
+  context.home = strdup(urldirname(start->url));
   context.home_length = strlen(context.home);
 
   noheuristics(&context);
@@ -1533,84 +1537,90 @@ static void ftp_body_parse(webbase_url_t* webbase_url, uri_t* url_object, char*
 #undef BUFFER_SIZE
 }
 
+static char* mirror_ftp_host = 0;
+
+static void mirror_ftp_free()
+{
+    if(mirror_ftp_host != 0) {
+	free(mirror_ftp_host);
+	ftpQuit();
+    }
+}
+
 static int mirror_ftp(crawl_context_t* context, webbase_url_t* webbase_url, uri_t* url_object, char* url)
 {
-  static char* host = 0;
-  int url_length = strlen(url);
+    int url_length = strlen(url);
 
-  webbase_url->w_info |= WEBBASE_URL_INFO_FTP;
+    webbase_url->w_info |= WEBBASE_URL_INFO_FTP;
 
-  if(host == 0 || strcmp(host, url_object->host)) {
-    if(host != 0) {
-      free(host);
-      ftpQuit();
-    }
+    if(mirror_ftp_host == 0 || strcmp(mirror_ftp_host, url_object->host)) {
+	mirror_ftp_free();
 
-    host = strdup(url_object->host);
+	mirror_ftp_host = strdup(url_object->host);
     
-    if(!ftpOpen(url_object->host)) {
-      fprintf(stderr, "mirror_ftp: %s cannot connect to %s\n", url, url_object->host);
-      return ECONNREFUSED;
+	if(!ftpOpen(url_object->host)) {
+	    fprintf(stderr, "mirror_ftp: %s cannot connect to %s\n", url, url_object->host);
+	    return ECONNREFUSED;
+	}
+	{
+	    char* user = url_object->user ? url_object->user : "ftp";
+	    char* passwd = url_object->passwd ? url_object->passwd : "someone@somewhere.com";
+	    if(!ftpLogin(user, passwd)) {
+		fprintf(stderr, "mirror_ftp: %s cannot login to %s with %s:%s\n", url, url_object->host, user, passwd);
+		webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_UNAUTHORIZED);
+		return ECONNREFUSED;
+	    }
+	}
     }
+
     {
-      char* user = url_object->user ? url_object->user : "ftp";
-      char* passwd = url_object->passwd ? url_object->passwd : "someone@somewhere.com";
-      if(!ftpLogin(user, passwd)) {
-	fprintf(stderr, "mirror_ftp: %s cannot login to %s with %s:%s\n", url, host, user, passwd);
-	webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_UNAUTHORIZED);
-	return ECONNREFUSED;
-      }
-    }
-  }
+	int isdir;
+	char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
+	int code;
 
-  {
-    int isdir;
-    char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
-    int code;
+	if(path) {
+	    creatp(path);
+	} else {
+	    fprintf(stderr, "mirror_ftp: cannot cache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
+	    webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_BAD_REQUEST);
+	    return ETIME;
+	}
 
-    if(path) {
-      creatp(path);
-    } else {
-      fprintf(stderr, "mirror_ftp: cannot cache %s, unable to resolve path (ignored)\n", webbase_url->w_url);
-      webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_BAD_REQUEST);
-      return ETIME;
+	isdir = (url[url_length - 1] == '/' || url[0] == '\0') ? 1 : 0;
+
+	if(isdir) 
+	    code = ftpDir(path, (url_object->path[0] == '\0' ? "/" : url_object->path));
+	else
+	    code = ftpGet(path, url_object->path, 'I');
+
+	if(!code) {
+	    fprintf(stderr, "mirror_ftp: %s cannot get/ls `%s'\n", url, url_object->path);
+	    webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_NOT_FOUND);
+	    return ETIME;
+	}
+
+	webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_OK);
+	if(isdir)
+	    webbase_url->w_info |= WEBBASE_URL_INFO_FTP_DIR;
+	webbase_url->w_info |= WEBBASE_URL_INFO_FTP | WEBBASE_URL_INFO_CONTENT;
+	webbase_url->w_content_length = file_size(path);
+	/*
+	 * Until better handling (parse directory containing file to find
+	 * the date of the file) the last modified time is the date of the
+	 * last successfull crawl.
+	 * If touching (only reload the file) do not change the date.
+	 */
+	if(!context->touch)
+	    webbase_url->w_mtime = time(0);
+	if(isdir) {
+	    strcpy(webbase_url->w_content_type, "text/ftp-dir-listing");
+	    ftp_body_parse(webbase_url, url_object, path);
+	} else {
+	    strcpy(webbase_url->w_content_type, mime_ext2mime(context->base, url_object->path, "text/plain"));
+	}
     }
 
-    isdir = (url[url_length - 1] == '/' || url[0] == '\0') ? 1 : 0;
-
-    if(isdir) 
-      code = ftpDir(path, (url_object->path[0] == '\0' ? "/" : url_object->path));
-    else
-      code = ftpGet(path, url_object->path, 'I');
-
-    if(!code) {
-      fprintf(stderr, "mirror_ftp: %s cannot get/ls `%s'\n", url, url_object->path);
-      webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_NOT_FOUND);
-      return ETIME;
-    }
-
-    webbase_url_code_set(webbase_url, WEBBASE_URL_CODE_OK);
-    if(isdir)
-      webbase_url->w_info |= WEBBASE_URL_INFO_FTP_DIR;
-    webbase_url->w_info |= WEBBASE_URL_INFO_FTP | WEBBASE_URL_INFO_CONTENT;
-    webbase_url->w_content_length = file_size(path);
-    /*
-     * Until better handling (parse directory containing file to find
-     * the date of the file) the last modified time is the date of the
-     * last successfull crawl.
-     * If touching (only reload the file) do not change the date.
-     */
-    if(!context->touch)
-      webbase_url->w_mtime = time(0);
-    if(isdir) {
-      strcpy(webbase_url->w_content_type, "text/ftp-dir-listing");
-      ftp_body_parse(webbase_url, url_object, path);
-    } else {
-      strcpy(webbase_url->w_content_type, mime_ext2mime(context->base, url_object->path, "text/plain"));
-    }
-  }
-
-  return 0;
+    return 0;
 }
 
 static int mirror_http(crawl_context_t* context, webbase_url_t* webbase_url, uri_t* url_object, char* url)
@@ -1882,13 +1892,14 @@ void crawl_free(crawl_params_t* params)
 {
   if(!params->no_hook)
     hook_end();
+  dirsel_end();
   robots_free(params->robots);
   cookies_free(params->cookies);
   hash_free(params->options);
   webbase_free(params->base);
   webtools_free(params->http);
+  mirror_ftp_free();
   free(params);
-  dirsel_end();
 }
 
 static void hnode_free(hnode_t *node, void *context)
