@@ -32,6 +32,9 @@
 #include <sys/types.h>
 #include <string.h>
 #include <fcntl.h>
+#ifdef HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif /* HAVE_DMALLOC_H */
 
 #include <hash.h>
 #include <webbase.h>
@@ -179,6 +182,7 @@ webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
 	sleep(3);
       } else {
 	error = 0;
+	break;
       }
     }
     if(error) {
