@@ -20,6 +20,9 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
+#ifdef HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif /* HAVE_DMALLOC_H */
 
 #include <sqlutil.h>
 #include <webbase_create.h>
