@@ -13,17 +13,24 @@
  * The copyright extends to translations of this work into other languages,
  * including machine languages. 
  *
- * $Id: hash.c,v 1.1.1.1 1999/07/12 07:27:31 loic Exp $
+ * $Id: hash.c,v 1.2 1999/07/19 12:39:15 loic Exp $
  * $Name:  $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdlib.h>
 #include <stddef.h>
 #include <assert.h>
+#ifdef HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif /* HAVE_DMALLOC_H */
 #define HASH_IMPLEMENTATION
 #include "hash.h"
 
-static const char rcsid[] = "$Id: hash.c,v 1.1.1.1 1999/07/12 07:27:31 loic Exp $";
+static const char rcsid[] = "$Id: hash.c,v 1.2 1999/07/19 12:39:15 loic Exp $";
 static const char right[] = "Copyright (C) 1997 Kaz Kylheku";
 
 #define INIT_BITS	6
